// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_kit/src/components/gradients/gradient_box_border.dart';
import 'package:wyatt_ui_kit/src/domain/card_theme_extension.dart';

class CardWrapper extends StatefulWidget {
  const CardWrapper({
    required this.child,
    required this.backgroundColors,
    required this.borderColors,
    required this.shadow,
    required this.minSize,
    required this.maxSize,
    required this.padding,
    required this.background,
    super.key,
  });

  final Widget? background;
  final Widget child;
  final List<Color>? backgroundColors;
  final List<Color>? borderColors;
  final BoxShadow? shadow;
  final Size? minSize;
  final Size? maxSize;
  final double? padding;

  @override
  State<CardWrapper> createState() => _CardWrapperState();
}

class _CardWrapperState extends State<CardWrapper> {
  Size _cardSize = Size.zero;
  final GlobalKey _key = GlobalKey();

  @override
  void initState() {
    super.initState();
    if (widget.background != null) {
      WidgetsBinding.instance.addPostFrameCallback((_) {
        _resizeCard();
      });
    }
  }

  @override
  void didUpdateWidget(covariant CardWrapper oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.background != null) {
      _resizeCard();
    }
  }

  void _resizeCard() {
    final RenderObject? renderBox = _key.currentContext?.findRenderObject();
    if (renderBox != null) {
      setState(() {
        _cardSize =
            Size(renderBox.paintBounds.width, renderBox.paintBounds.height);
      });
    }
  }

  Widget _buildChild(Widget child) => (widget.background != null)
      ? Stack(
          alignment: Alignment.center,
          children: [
            SizedBox(
              width: _cardSize.width,
              height: _cardSize.height,
              child: Center(child: widget.background),
            ),
            Padding(
              padding: EdgeInsets.all(
                widget.padding ?? 25,
              ),
              child: child,
            ),
          ],
        )
      : Padding(
          padding: EdgeInsets.all(
            widget.padding ?? 25,
          ),
          child: child,
        );

  Gradient? _cardGradient(BuildContext context) {
    if (widget.backgroundColors != null &&
        widget.backgroundColors!.length >= 2) {
      return LinearGradient(colors: widget.backgroundColors!);
    } else {
      final extensionCardColor =
          Theme.of(context).extension<CardThemeExtension>();

      if (extensionCardColor != null &&
          extensionCardColor.backgroundColors != null &&
          extensionCardColor.backgroundColors!.length >= 2) {
        return LinearGradient(colors: extensionCardColor.backgroundColors!);
      }
    }
    return null;
  }

  Color? _cardColor(BuildContext context) {
    if (widget.backgroundColors != null &&
        widget.backgroundColors!.length == 1) {
      return widget.backgroundColors!.first;
    } else {
      final extensionCardColor =
          Theme.of(context).extension<CardThemeExtension>();

      if (extensionCardColor != null &&
          extensionCardColor.backgroundColors != null &&
          extensionCardColor.backgroundColors!.length == 1) {
        return extensionCardColor.backgroundColors!.first;
      }
    }
    return Theme.of(context).cardColor;
  }

  BoxBorder? _boxBorder(BuildContext context) {
    if (widget.borderColors != null) {
      if (widget.borderColors!.length >= 2) {
        return GradientBoxBorder(
          gradient: LinearGradient(
            colors: widget.borderColors!,
          ),
        );
      } else if (widget.borderColors!.isNotEmpty) {
        return Border.all(
          color: widget.borderColors!.first,
        );
      }
    } else {
      final extensionCardColor =
          Theme.of(context).extension<CardThemeExtension>();
      if (extensionCardColor != null &&
          extensionCardColor.borderColor != null) {
        if (extensionCardColor.borderColor!.length >= 2) {
          return GradientBoxBorder(
            gradient: LinearGradient(
              colors: extensionCardColor.borderColor!,
            ),
          );
        } else if (extensionCardColor.backgroundColors!.isNotEmpty) {
          return Border.all(
            color: extensionCardColor.backgroundColors!.first,
          );
        }
      }
    }
    return null;
  }

  List<BoxShadow> _shadow(BuildContext context) {
    final shadows = List<BoxShadow>.empty(growable: true);
    if (widget.shadow != null) {
      shadows.add(widget.shadow!);
    } else {
      final extensionCardColor =
          Theme.of(context).extension<CardThemeExtension>();
      if (extensionCardColor != null &&
          extensionCardColor.shadowColor != null) {
        shadows.add(extensionCardColor.shadowColor!);
      }
    }
    return shadows;
  }

  @override
  Widget build(BuildContext context) => SelectionArea(
        child: DecoratedBox(
          key: _key,
          decoration: BoxDecoration(
            borderRadius: const BorderRadius.all(Radius.circular(12)),
            gradient: _cardGradient(context),
            color: _cardColor(context),
            border: _boxBorder(context),
            boxShadow: _shadow(context),
          ),
          child: (widget.minSize != null && widget.maxSize != null)
              ? ConstrainedBox(
                  constraints: BoxConstraints(
                    minWidth: widget.minSize!.width,
                    minHeight: widget.minSize!.height,
                    maxWidth: widget.maxSize!.width,
                    maxHeight: widget.maxSize!.height,
                  ),
                  child: _buildChild(widget.child),
                )
              : _buildChild(widget.child),
        ),
      );
}
